unit Aliens;

interface

uses
  windows,
  level,
  weaponry,
  turrican,
  Core_Graphic,
  Core_Sound;

const
  BONUS_LASER = $01;
  BONUS_ENERGY = $02;
  MAX_BONUS_TYPE = $02;

type
  TAlien = class
    X, Y : Word;
    Width, Height : Byte;
    Energy : Smallint;
    Enabled : Boolean;
    dead : Boolean;
  protected
    Procedure Init(Xpos, Ypos : integer); virtual; abstract;
    procedure process; virtual; abstract;
    procedure death; virtual; abstract;
    procedure hit; virtual; abstract;
    procedure Draw(Xpos, Ypos : integer); virtual; abstract;
  end;

  TBonusFlyer = class(TAlien)
    Xs, Ys : Word;
    cycle : Word;
  protected
    Procedure Init(Xpos, Ypos : integer); override;
    procedure process; override;
    procedure death; override;
    procedure hit; override;
    procedure Draw(Xpos, Ypos : integer); override;
  end;

  TDroneBomb = class(TAlien)
    cycle : byte;
  protected
    Procedure Init(Xpos, Ypos : integer); override;
    procedure process; override;
    procedure death; override;
    procedure hit; override;
    procedure Draw(Xpos, Ypos : integer); override;
  end;

  TBonus = class(TAlien)
    BonusType : byte;
  protected
    Procedure Init(Xpos, Ypos : integer); override;
    procedure process; override;
    procedure death; override;
    procedure hit; override;
    procedure Draw(Xpos, Ypos : integer); override;
  end;

  TBoss = class(TAlien)
    BonusType : byte;
  protected
    StartX : Integer;
    StartY : Integer;
    MoveX, MoveY : Integer;
    Xinc : integer;
    Search : Boolean;
    Atack : Boolean;
    Procedure Init(Xpos, Ypos : integer); override;
    procedure process; override;
    procedure death; override;
    procedure hit; override;
    procedure Draw(Xpos, Ypos : integer); override;
  end;

  TEnemyEngine = object
    EnemyCount : Word;
    Enemies : array of TAlien;
    procedure Init(lvl : TLevel);
    procedure Process(Xpos, YPos : Integer);
    procedure Draw(Xpos, YPos : Integer);
    procedure CollideWithPlayer(rect : TRect);
    procedure CollideWithShoots(weapon : Tweapon);
  end;

implementation

uses defs, mainengine;

procedure aft2();
begin
  player.Lives:= 0;
  player.Death;
  startmusic('mdat.T2_Title');
end;


//=============================================================================
// EnemyEngine Init
//=============================================================================
procedure TEnemyEngine.Init(lvl : TLevel);
var
  i : Integer;
begin
  if EnemyCount > 0 then
  begin
    for i:=0 to EnemyCount-1 do
    Enemies[i].Free;
  end;

  EnemyCount:= 0;
  SetLength(Enemies, 0);

  for i:=0 to lvl.EnemyCount-1 do
  begin
    inc(EnemyCount);
    SetLength(Enemies, EnemyCount);

    case lvl.Enemies[i].Enemy of
      ENEMY_BONYSFLYER : Enemies[i]:= TBonusFlyer.Create;
      ENEMY_DRONEBOMB  : Enemies[i]:= TDroneBomb.Create;
      ENEMY_BOSS       : Enemies[i]:= TBoss.Create;
    end;

    Enemies[i].Init(lvl.enemies[i].X, lvl.enemies[i].Y);
  end;
end;

//=============================================================================
// Process. Move all enemies
//=============================================================================
procedure TEnemyEngine.Process(Xpos, YPos : Integer);
var
  i : integer;
begin
  if EnemyCount = 0 then exit;

 //  
  i:= 0;
  repeat
    if Enemies[i].dead then
    begin
      if Enemies[i] is TBonusFlyer then
      begin
        inc(EnemyCount);
        SetLength(Enemies, EnemyCount);
        Enemies[EnemyCount-1]:= TBonus.Create;
        Enemies[EnemyCount-1].Init(Enemies[i].X, Enemies[i].Y);
      end;

      dec(EnemyCount);
      Enemies[i].Death;
      Enemies[i].Free;
      Enemies[i]:= Enemies[EnemyCount];
      SetLength(Enemies, EnemyCount);
      dec(i);
    end;
    inc(i);
  until i = EnemyCount;

  if EnemyCount = 0 then exit;

  for i:=0 to EnemyCount-1 do
  begin
    Enemies[i].Enabled:= true;
    if ((Enemies[i].X + Xpos) < -50) or ((Enemies[i].X + Xpos) > 370) or
       ((Enemies[i].Y + Ypos) < -30) or ((Enemies[i].Y + Ypos) > 270) then
    Enemies[i].Enabled:= false
    else Enemies[i].process;
  end;
end;

//=============================================================================
// Draw all enemies
//=============================================================================
procedure TEnemyEngine.Draw(Xpos, YPos : Integer);
var
  i : integer;
  NewX, NewY : Integer;
begin
  if EnemyCount = 0 then exit;

  for i:=0 to EnemyCount-1 do
  if Enemies[i].Enabled then
  begin
    NewX:= Enemies[i].X + Xpos;
    NewY:= Enemies[i].Y + Ypos;
    Enemies[i].Draw(NewX, NewY);
  end;
end;

//=============================================================================
// enemy collide with player
//=============================================================================
procedure TEnemyEngine.CollideWithPlayer(rect : TRect);
var
  i : integer;
  res, r : TRect;
begin
  if EnemyCount = 0 then exit;

  for i:=0 to EnemyCount-1 do
  if Enemies[i].Enabled then
  begin
    SetRect(r, enemies[i].X, enemies[i].Y, enemies[i].X + enemies[i].Width, enemies[i].Y + enemies[i].Height);
    if not IntersectRect(res, r, rect) then exit;
    enemies[i].Hit;
  end;
end;

//=============================================================================
// enemy collide with shoots
//=============================================================================
procedure TEnemyEngine.CollideWithShoots(weapon : Tweapon);
var
  i, j : integer;
  res, shoot, en : TRect;
begin
  if EnemyCount = 0 then exit;
  if weapon.ShootCount = 0 then Exit;

  for i:=0 to EnemyCount-1 do
  if Enemies[i].Enabled then
  begin
    if Enemies[i] is TBonus then Continue;

    SetRect(en, enemies[i].X, enemies[i].Y, enemies[i].X + enemies[i].Width, enemies[i].Y + enemies[i].Height);

    for j:= 0 to weapon.ShootCount-1 do
    if not weapon.Shoots[j].dead then
    begin
      SetRect(shoot, weapon.Shoots[j].X, weapon.Shoots[j].Y, weapon.Shoots[j].X + 30, weapon.Shoots[j].Y + 30);
      SetRect(en, enemies[i].X, enemies[i].Y, enemies[i].X + enemies[i].Width, enemies[i].Y + enemies[i].Height);
      if IntersectRect(res, en, shoot) then
      begin
        weapon.Shoots[j].dead:= true;
        Enemies[i].Energy:= Enemies[i].Energy - weapon.damage[weapon.Level];
        if Enemies[i].Energy <= 0 then Enemies[i].dead:= true;
        Core_Sound_Play(sExplosion);
      end;
    end;
  end;
end;

//=============================================================================
//
//=============================================================================
procedure TBonusFlyer.Init(Xpos, Ypos : integer);
begin
  X:= Xpos;
  Y:= Ypos;
  Ys:= Y;
  Xs:= X;
  Width:= 24;
  Height:= 15;
  cycle:= random(360);
  Energy:= 15;
  dead:= false;
end;

//=============================================================================
//
//=============================================================================
procedure TBonusFlyer.process();
begin
  inc(cycle);
  if cycle = 360 then cycle:= 0;
  Y:= trunc(Ys + Sin((Cycle*8*Pi)/180)*40);
  X:= trunc(Xs + Cos((Cycle*Pi)/180)*100);
end;

//=============================================================================
//
//=============================================================================
procedure TBonusFlyer.Draw(Xpos, Ypos : integer);
begin
  Core_Graphic_DrawSprite(Xpos, Ypos, bonusflyer, nil);
end;

//=============================================================================
//
//=============================================================================
procedure TBonusFlyer.death();
begin
  player.Score:= player.Score + 150;
  Core_Sound_Play(sExplosion2);
end;

//=============================================================================
//
//=============================================================================
procedure TBonusFlyer.hit();
begin
  player.Energy:= player.Energy - 2;
  Core_Sound_Play(sHit);
end;

//=============================================================================
//
//=============================================================================
procedure TDroneBomb.Init(Xpos, Ypos : integer);
begin
  X:= Xpos;
  Y:= Ypos;
  Width:= 15;
  Height:= 22;
  cycle:= 0;
  Energy:= 10;
  dead:= false;
end;

//=============================================================================
//
//=============================================================================
procedure TDroneBomb.process();
begin
  inc(cycle);
  if cycle = 3 then cycle:= 0;
  Y:= Y + 2;
end;

//=============================================================================
//
//=============================================================================
procedure TDroneBomb.Draw(Xpos, Ypos : integer);
var
  rect : TRect;
begin
  SetRect(rect, cycle*15, 0, cycle*15+15, 22);
  Core_Graphic_DrawSprite(Xpos, Ypos, dronebomb, @rect);
end;

//=============================================================================
//
//=============================================================================
procedure TDroneBomb.death();
begin
  player.Score:= player.Score + 70;
  Core_Sound_Play(sExplosion2);
end;

//=============================================================================
//
//=============================================================================
procedure TDroneBomb.hit();
begin
  player.Energy:= player.Energy - 2;
  Core_Sound_Play(sHit);
end;

//=============================================================================
//
//=============================================================================
procedure TBonus.Init(Xpos, Ypos : integer);
begin
  X:= Xpos;
  Y:= Ypos;
  Width:= 14;
  Height:= 13;
  BonusType:= random(MAX_BONUS_TYPE) + 1;
  dead:= false;
end;

//=============================================================================
//
//=============================================================================
procedure TBonus.process();
begin
  Y:= Y - 1;
end;

//=============================================================================
//
//=============================================================================
procedure TBonus.Draw(Xpos, Ypos : integer);
var
  rect : TRect;
begin
  case BonusType of
    BONUS_ENERGY : SetRect(rect, 0, 0, 14, 13);
    BONUS_LASER  : SetRect(rect, 28, 0, 42, 13);
  end;
  Core_Graphic_DrawSprite(Xpos, Ypos, Bonuses, @rect);
end;

//=============================================================================
//
//=============================================================================
procedure TBonus.hit();
begin
  player.Score:= player.Score + 100;
  case BonusType of
    BONUS_ENERGY : begin
                     Core_Sound_Play(sEnergy);
                     player.Energy:= MAX_ENERGY;
                   end;
    BONUS_LASER  : begin
                     Core_Sound_Play(sLaser);
                     if player.Weapon is Tlaser then
                     begin
                       if player.Weapon.Level < 4 then player.Weapon.Level:= player.Weapon.Level + 1;
                     end else
                     begin
                       weapon_laser.Level:= 0;
                       player.Weapon:= weapon_laser;
                     end;
                   end;
  end;
  dead:= true;
end;

//=============================================================================
//
//=============================================================================
procedure TBonus.death();
begin

end;

//=============================================================================
//
//=============================================================================
procedure TBoss.Init(Xpos, Ypos : integer);
begin
  X:= Xpos;
  Y:= Ypos;
  StartY:= Y;
  StartX:= X;
  Width:= 120;
  Height:= 74;
  Energy:= 200;
  Search:= true;
  Atack:= false;
  Xinc:= -5;
  MoveX:= X - 100;
end;

//=============================================================================
//
//=============================================================================
procedure TBoss.process();
begin
  if Search then
  begin
    if abs(X - MoveX) < 8 then begin
                                 Atack:= true;
                                 Search:= false;
                                 MoveY:= player.Y - 30;
                               end else X:= X + Xinc;
  end else
  begin
    if Atack then
    begin
      if abs(Y - MoveY) < 8 then Atack:= False
                            else Y:= Y + 5;
    end else
    begin
      if abs(Y - StartY) < 8 then
      begin
        Search:= true;
        MoveX:= player.X-30;
        if X < player.X then Xinc:= 3
                        else Xinc:= -3;
      end else Y:= Y - 3;
    end;
  end;
  if abs(X - player.X) > 200 then Init(StartX, StartY);
end;

//=============================================================================
//
//=============================================================================
procedure TBoss.Draw(Xpos, Ypos : integer);
var
  rect : TRect;
begin
  setrect(rect, 0, 0, 120, 74);
  Core_Graphic_DrawSprite(Xpos, Ypos, BOSS, @rect);
end;

//=============================================================================
//
//=============================================================================
procedure TBoss.hit();
begin
  player.Energy:= player.Energy - 4;
  Core_Sound_Play(sHit);
end;

//=============================================================================
//
//=============================================================================
procedure TBoss.death();
begin
  player.Score:= player.Score + 5000;
  Core_Sound_Play(sExplosion2);
  Core_Sound_Play(sLevelDone);
  afterpause:= aft2;
  EndMusic;
  pause:= 60;
  levelNo:= 1;
end;


end.

