unit Core_Graphic;

interface

uses
  Windows,
  DirectDraw,
  DDUtil;

const
  screenx = 320;            //    
  screeny = 240;            //    

var
  DD : IDirectDraw7;
  DDSPrimary : IDirectDrawSurface7;
  DDSSecondary : IDirectDrawSurface7;

  //    
  DGamma : IDirectDrawGammaControl;
  OldGamma, NewGamma : TDDGammaRamp;

  //     
  FadeO : Boolean = false;
  FadeI  : Boolean = false;
  FadeTime : Byte = 64;

procedure Core_Graphic_Init(Handle : HWND);
procedure Core_Graphic_InitBackBuffer();
procedure Core_Graphic_DrawSprite(x, y : Integer; sprite : IDirectDrawSurface7; rect : PRect);
procedure Core_Graphic_InitGamma();
Procedure Core_Graphic_FadeOut();
Procedure Core_Graphic_FadeIn();
procedure Core_Graphic_LoadImage(filename : Pchar; var surface : IDirectDrawSurface7; ColorKey : Cardinal);
procedure Core_Graphic_Clear (R, G, B : Byte);
procedure Core_Graphic_Fill (R, G, B : Byte; rect : PRect);

implementation

//=============================================================================
// DirectDraw Init
//=============================================================================
procedure Core_Graphic_Init(Handle : HWND);
var
  ddsd : TDDSurfaceDesc2;
  hret : HRESULT;
begin
  DD:= nil;
  DDSPrimary:= nil;
  DDSSecondary:= nil;

  // DirectDraw
  hret:= DirectDrawCreateEx(nil, DD, IDirectDraw7, nil);
  if Failed (hRet) then MessageBox(Handle, '', 'DirectDrawCreateEx', MB_ICONERROR);

  hret:= DD.SetCooperativeLevel(Handle, DDSCL_EXCLUSIVE or DDSCL_FULLSCREEN);
  if Failed (hRet) then MessageBox(Handle, '', 'SetCooperativeLevel', MB_ICONERROR);

  hret:= DD.SetDisplayMode(320, 240, 16, 0, 0);
  if Failed (hRet) then MessageBox(Handle, '', 'SetDisplayMode', MB_ICONERROR);

  FillChar(ddsd, sizeOf(ddsd), 0);
  with ddsd do
  begin
    dwSize:= sizeof(ddsd);
    dwFlags:= DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
    ddsCaps.dwCaps:= DDSCAPS_PRIMARYSURFACE or DDSCAPS_FLIP or DDSCAPS_COMPLEX;
    dwBackBufferCount:= 1;
  end;

  hret:= DD.CreateSurface(ddsd, DDSPrimary, nil);
  if Failed (hRet) then MessageBox(Handle, '', 'CreateSurface', MB_ICONERROR);
end;

//=============================================================================
// DirectDraw Backbuffer Init
//=============================================================================
procedure Core_Graphic_InitBackBuffer();
var
  ddscaps : TDDSCaps2;
begin
  // 
  ZeroMemory (@ddscaps, SizeOf(ddscaps));
  ddsCaps.dwCaps:= DDSCAPS_BACKBUFFER;

  DDSPrimary.GetAttachedSurface(ddscaps, DDSSecondary);
end;

//=============================================================================
//     
//=============================================================================
procedure Core_Graphic_DrawSprite(x, y : Integer; sprite : IDirectDrawSurface7; rect : PRect);
var
  dx, dy : Integer;
  rec : TRect;
begin
  //  
  if rect <> nil then
  begin
    SetRect(rec, rect^.left, rect^.top, rect^.right, rect^.bottom);

    if x < 0 then begin rec.Left:= rect^.Left - x; x:= 0; end; //     
    if y < 0 then begin rec.Top:= rect^.Top - y; y:= 0; end;   //   

    dx:= x + (rect^.Right - rect^.Left) - screenx;
    dy:= y + (rect^.Bottom - rect^.Top) - screeny;
    if dx > 0 then rec.Right:= rect^.Right - dx;
    if dy > 0 then rec.Bottom:= rect^.Bottom - dy;

    //    
    DDSSecondary.BltFast(x, y, sprite, @rec, DDBLTFAST_WAIT or DDBLTFAST_SRCCOLORKEY);
  end else
  //    
  DDSSecondary.BltFast(x, y, sprite, nil, DDBLTFAST_WAIT or DDBLTFAST_SRCCOLORKEY);
end;

//=============================================================================
//   
//=============================================================================
procedure Core_Graphic_InitGamma();
begin
  DGamma:= nil;
  //     
  DDSPrimary.QueryInterface(IID_IDirectDrawGammaControl, DGamma);
  //   
  DGamma.GetGammaRamp(0, OldGamma);
  DGamma.GetGammaRamp(0, NewGamma);
end;

//=============================================================================
// FadeOut
//=============================================================================
Procedure Core_Graphic_FadeOut();
var
  i : integer;
begin
  //  
  for i :=0 to 255 do
  begin
    NewGamma.red[i]:=Word((OldGamma.red[i]*FadeTime) shr 6);
    NewGamma.green[i]:=Word((OldGamma.green[i]*FadeTime) shr 6);
    NewGamma.blue[i]:=Word((OldGamma.blue[i]*FadeTime) shr 6);
  end;
  DGamma.SetGammaRamp(0,NewGamma);
  //         .
  dec(FadeTime);
  if FadeTime = 1 then
  begin
    FadeO:= false;
    FadeTime:= 64;
  end;
end;

//=============================================================================
// FadeIn
//=============================================================================
Procedure Core_Graphic_FadeIn();
var
  i : integer;
begin
  //  
  for i :=0 to 255 do
  begin
    NewGamma.red[i]:=Word((OldGamma.red[i]*(64-FadeTime)) shr 6);
    NewGamma.green[i]:=Word((OldGamma.green[i]*(64-FadeTime)) shr 6);
    NewGamma.blue[i]:=Word((OldGamma.blue[i]*(64-FadeTime)) shr 6);
  end;
  DGamma.SetGammaRamp(0,NewGamma);
  //         .
  dec(FadeTime);
  if FadeTime = 1 then
  begin
    FadeI:= false;
    FadeTime:= 64;
  end;
end;

//=============================================================================
//        
//=============================================================================
procedure Core_Graphic_LoadImage(filename : Pchar; var surface : IDirectDrawSurface7; ColorKey : Cardinal);
begin
  surface:= DDLoadBitmap(DD, filename, 0, 0);
  DDSetColorKey(surface, ColorKey);
end;

//=============================================================================
//        :)
//=============================================================================
procedure Core_Graphic_Clear (R, G, B : Byte);
var
  ddbltfx : TDDBLTFX;
begin
  ZeroMemory(@ddbltfx, SizeOf(ddbltfx));
  ddbltfx.dwSize := SizeOf(ddbltfx);
  ddbltfx.dwFillColor := RGB (R, G, B);
  DDSSecondary.Blt(nil, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @ddbltfx);
end;

//=============================================================================
//  
//=============================================================================
procedure Core_Graphic_Fill (R, G, B : Byte; rect : PRect);
var
  ddbltfx : TDDBLTFX;
begin
  ZeroMemory(@ddbltfx, SizeOf(ddbltfx));
  ddbltfx.dwSize := SizeOf(ddbltfx);
  ddbltfx.dwFillColor := RGB (R, G, B);
  DDSSecondary.Blt(rect, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @ddbltfx);
end;



end.
