unit Core_Sound;

interface

uses
  Windows,
  DirectSound,
  mmSystem;

type
  TWaveFileHeader = packed record
    FileDesc : array [0..3] of char;   // Must be 'RIFF'
    FileSize : Cardinal;               // FileSize
    WaveDesc : array [0..3] of char;   // Must be 'WAVE'
    FMTdesc  : array [0..3] of char;   // Must be 'fmt '
    WaveSize : Cardinal;               // Wave Size
    Wavefmt  : Word;                   // $01 for non compressed wave
    Channels : Word;                   // $01 -mono, $02 - stereo and so on
    Samples  : Cardinal;               // Hz
    BPS      : Cardinal;               // Number_of_channels*Samples_per_second*Bits_per_Sample/8
    Align    : Word;                   // Number_of_channels*Bits_per_Sample/8
    Bits     : Word;                   //
    DataDesc : array [0..3] of char;   // Must be 'data'
    DataSize : Cardinal;               // Size of Data
  end;

var
  DSound : IDirectSound8;
  DSBPrimary : IDirectSoundBuffer;

procedure Core_Sound_Init(Handle : HWND);
procedure Core_Sound_LoadWave(Filename : String; var buffer : IDirectSoundBuffer);
procedure Core_Sound_Play(sound : IDirectSoundBuffer);

implementation

//=============================================================================
// DirectSound Init
//=============================================================================
procedure Core_Sound_Init(Handle : HWND);
var
  Bufferdesc : TDSBufferDesc;
  BFormat : TWaveFormatEx;
  hret : HRESULT;
begin
  DSound:= nil;

  hret:= DirectSoundCreate8(nil, Dsound, nil);
  if hret <> DS_OK then exit;

  hret:= DSound.SetCooperativeLevel(Handle, DSSCL_NORMAL);
  if hret <> DS_OK then exit;

{  ZeroMemory(@Bufferdesc, SizeOf(Bufferdesc));
  with Bufferdesc do
  begin
    dwSize:= sizeOf(TDSBufferDesc);
    dwFlags:= DSBCAPS_PRIMARYBUFFER;
    dwBufferBytes:= 0;
    lpwfxFormat:= nil;
  end;

  hret:= DSound.CreateSoundBuffer(Bufferdesc, DSBPrimary, nil);
  if hret <> DS_OK then exit;

  ZeroMemory(@BFormat, SizeOf(TWaveFormatEx));
  with BFormat do
  begin
    wFormatTag:= WAVE_FORMAT_PCM;
    nChannels:= 2;
    nSamplesPerSec:= 22050;
    nBlockAlign:= 4;
    nAvgBytesPerSec:= nSamplesPerSec * nBlockAlign;
    wBitsPerSample:= 16;

    cbSize:= 0;
  end;

  hret:= DSBPrimary.SetFormat(@BFormat);
  if hret <> DS_OK then exit;

  Sound_Channels:= Sound_Channels_Max;   }
end;

//=============================================================================
// DirectSound Load wave
//=============================================================================
procedure Core_Sound_LoadWave(Filename : String; var buffer : IDirectSoundBuffer);
var
  f : file;
  Header : TWaveFileHeader;
  bufferdesc : TDSBufferDesc;
  waveformat : TWaveFormatEx;

  data, data2 : Pointer;
  datasize, datasize2 : Cardinal;

  hret : HRESULT;
begin
  AssignFile(f, filename);
  reset(f, 1);
  BlockRead(f, Header, SizeOf(TWaveFileHeader));

  // if file NOT wave
  if (Header.FileDesc <> 'RIFF') or (Header.WaveDesc<>'WAVE') then
  begin
    CloseFile(f);
    Exit;
  end;

  if Header.DataDesc <> 'data' then
  repeat
    GetMem(data2, Header.DataSize);
    BlockRead(f, data2^, Header.DataSize);
    BlockRead(f, Header.DataDesc, 4);
    BlockRead(f, Header.DataSize, 4);
    FreeMem(data2);
  until Header.DataDesc = 'data';

  // fill waveformat data
  waveformat.cbSize:= 0;
  waveformat.wFormatTag:= Header.Wavefmt;
  waveformat.nChannels:= Header.Channels;
  waveformat.nSamplesPerSec:= Header.Samples;
  waveformat.wBitsPerSample:= Header.Bits;
  waveformat.nBlockAlign:= Header.Bits div 8 * Header.Channels;
  waveformat.nAvgBytesPerSec:= Header.Samples * waveformat.nBlockAlign;

  // fill buffer description
  ZeroMemory(@bufferdesc, SizeOf(TDSBufferDesc));
  with bufferdesc do
  begin
    dwSize:= SizeOf(TDSBufferDesc);
    dwFlags:= DSBCAPS_STATIC or DSBCAPS_CTRLVOLUME;
    dwBufferBytes:= Header.DataSize;
    lpwfxFormat:= @waveformat;
  end;

  // Buffer Create
  hret:= DSound.CreateSoundBuffer(bufferdesc, buffer, nil);
  if hret <> DS_OK then exit;

  // fill audio data
  data:= nil;
  datasize:= 0;
  data2:= nil;
  datasize2:= 0;

{  hret:= buffer.Restore;
  if hret <> DS_OK then exit;
                                 }
  hret:= buffer.Lock(0, 0, @data, @datasize, @data2, @datasize2, DSBLOCK_ENTIREBUFFER);
  if hret <> DS_OK then exit;

  // read audio data
  BlockRead(f, data^, datasize);
  CloseFile(f);

  hret:= buffer.Unlock(data, datasize, data2, datasize2);
  if hret <> DS_OK then exit;

  hret:= buffer.SetVolume(-1000);
  if hret <> DS_OK then exit;
end;

//=============================================================================
// DirectSound Play sound
//=============================================================================
procedure Core_Sound_Play(sound : IDirectSoundBuffer);
var
  hret : HRESULT;
begin
  hret:= sound.SetCurrentPosition(0);
  if hret <> DS_OK then exit;

  hret:= sound.Play(0, 0, 0);
  if hret <> DS_OK then exit;
end;



end.
