unit level;

interface

uses
  Windows;

const
  // Triggers
  TRIGGER_RESPAWN  = $01;
  TRIGGER_DEATH    = $02;
  TRIGGER_ENDLEVEL = $03;

  // Enemies
  ENEMY_BONYSFLYER = $01;
  ENEMY_DRONEBOMB  = $02;
  ENEMY_BOSS       = $10;

type
  TTile = packed record
    X, Y : Byte;
    No : Byte;
    layer : Byte;
  end;

  TTrigger = packed record
    X1, X2, Y1, Y2 : Word;
    Trigger : byte;
  end;

  TCrystal = packed record
    X, Y : Word;
  end;

  TEnemy = packed record
    X, Y : Word;
    Enemy : Byte;
  end;

  TLevel = object
    RespawnX, RespawnY : Word;
    grid : array[0..127, 0..127] of Boolean;
    tilescount : Word;
    tiles : array of TTile;
    CrystalsCount : Word;
    Crystals : array of TCrystal;
    TriggerCount : Word;
    Triggers : array of TTrigger;
    TriggersRect : array of trect;
    EnemyCount : Word;
    Enemies : array of TEnemy;
    procedure AddCrystal(X, Y : Word);
    procedure AddTrigg(X, Y, XX, YY : Word; Trig : byte);
    procedure AddTile(X, Y, No, Layer : byte);
    procedure save(Filename : string);
    procedure load(Filename : string);
    procedure clear();
    procedure MakeGrid();
    procedure DeleteCrystal(index : integer);
    procedure AddEnemy(X, Y : Word; Enemy : byte);
  end;

implementation

//=============================================================================
//        index
//=============================================================================
procedure TLevel.DeleteCrystal(index : integer);
begin
  CrystalsCount:= CrystalsCount - 1;
  if index <> CrystalsCount then
  begin
    Crystals[index].X:= Crystals[CrystalsCount].X;
    Crystals[index].Y:= Crystals[CrystalsCount].Y;
  end;
  SetLength(Crystals, CrystalsCount);
end;

//=============================================================================
//    .
//=============================================================================
procedure TLevel.AddCrystal(X, Y : Word);
begin
  inc(CrystalsCount);
  SetLength(Crystals, CrystalsCount);
  Crystals[CrystalsCount-1].X:= X;
  Crystals[CrystalsCount-1].Y:= Y;
end;

//=============================================================================
//    .
//=============================================================================
procedure TLevel.AddEnemy(X, Y : Word; Enemy : byte);
begin
  inc(EnemyCount);
  SetLength(Enemies, EnemyCount);
  Enemies[EnemyCount-1].X:= X;
  Enemies[EnemyCount-1].Y:= Y;
  Enemies[EnemyCount-1].Enemy:= Enemy;
end;

//=============================================================================
//    .
//=============================================================================
procedure TLevel.AddTrigg(X, Y, XX, YY : Word; Trig : byte);
begin
  inc(TriggerCount);
  SetLength(Triggers, TriggerCount);
  Triggers[TriggerCount-1].X1:= X;
  Triggers[TriggerCount-1].Y1:= Y;
  Triggers[TriggerCount-1].X2:= XX;
  Triggers[TriggerCount-1].Y2:= YY;
  Triggers[TriggerCount-1].Trigger:= Trig;
end;

//=============================================================================
//  
//=============================================================================
procedure TLevel.AddTile(X, Y, No, Layer : byte);
begin
  inc(tilescount);
  SetLength(tiles, tilescount);
  tiles[tilescount-1].X:= X;
  tiles[tilescount-1].Y:= Y;
  tiles[tilescount-1].No:= No;
  tiles[tilescount-1].layer:= Layer;
end;

//=============================================================================
//          
//=============================================================================
procedure TLevel.MakeGrid();
var
  i, j : integer;
begin
  for i:=0 to 127 do
  for j:=0 to 127 do
  grid[i, j]:= false;

  for i:=0 to tilescount-1 do
  begin
    if tiles[i].layer = 4 then
    grid[tiles[i].X, tiles[i].Y]:= true;
  end;
end;

//=============================================================================
//    
//=============================================================================
procedure TLevel.load(filename : string);
var
  f : file;
  i : Integer;
begin
  AssignFile(f, filename);
  Reset(f, 1);
  BlockRead(f, tilescount, 2);
  SetLength(tiles, tilescount);
  BlockRead(f, tiles[0], sizeOf(ttile)*tilescount);

  BlockRead(f, CrystalsCount, 2);
  SetLength(Crystals, CrystalsCount);
  BlockRead(f, Crystals[0], sizeOf(tcrystal)*CrystalsCount);

  BlockRead(f, TriggerCount, 2);
  SetLength(Triggers, TriggerCount);
  BlockRead(f, Triggers[0], sizeOf(ttrigger)*TriggerCount);

  BlockRead(f, EnemyCount, 2);
  SetLength(Enemies, EnemyCount);
  BlockRead(f, Enemies[0], sizeOf(tEnemy)*EnemyCount);

  CloseFile(f);

  //    

  SetLength(TriggersRect, TriggerCount);
  if TriggerCount > 0 then
  for i:=0 to TriggerCount-1 do
  begin
    SetRect(TriggersRect[i], Triggers[i].X1, Triggers[i].Y1, Triggers[i].X2, Triggers[i].Y2);
    if Triggers[i].Trigger = TRIGGER_RESPAWN then
    begin
      RespawnX:= Triggers[i].X1;
      RespawnY:= Triggers[i].Y1;
    end;
  end;

  MakeGrid;
end;

//=============================================================================
//    
//=============================================================================
procedure TLevel.Save(filename : string);
var
  f : file;
begin
  AssignFile(f, filename);
  Rewrite(f, 1);
  BlockWrite(f, tilescount, 2);
  BlockWrite(f, tiles[0], sizeOf(ttile)*tilescount);
  BlockWrite(f, CrystalsCount, 2);
  BlockWrite(f, Crystals[0], sizeOf(tcrystal)*CrystalsCount);
  BlockWrite(f, TriggerCount, 2);
  BlockWrite(f, Triggers[0], sizeOf(ttrigger)*TriggerCount);
  BlockWrite(f, EnemyCount, 2);
  BlockWrite(f, Enemies[0], sizeOf(tEnemy)*EnemyCount);
  CloseFile(f);
end;

//=============================================================================
//     :)
//=============================================================================
procedure TLevel.clear;
begin
  SetLength(tiles, 0);
  tilescount:= 0;
  SetLength(Crystals, 0);
  CrystalsCount:= 0;
  SetLength(Triggers, 0);
  TriggerCount:= 0;
  SetLength(Enemies, 0);
  EnemyCount:= 0;
end;

end.
